Package: AppliedPredictiveModeling
Type: Package
Title: Functions and Data Sets for 'Applied Predictive Modeling'
Version: 1.1-7
Date: 2018-05-22
Author: Max Kuhn, Kjell Johnson
Maintainer: Max Kuhn <mxkuhn@gmail.com>
Description: A few functions and several data set for the Springer book 'Applied Predictive Modeling'.
URL: http://appliedpredictivemodeling.com/
Depends: R (>= 2.10)
Imports: CORElearn, MASS, plyr, reshape2, lattice, ellipse
Suggests: caret (>= 6.0-22)
License: GPL-2
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2018-05-22 19:03:00 UTC; max
Repository: CRAN
Date/Publication: 2018-05-22 19:14:21 UTC
Built: R 4.6.0; ; 2026-04-26 02:38:12 UTC; unix
