# Generated by roxygen2: do not edit by hand

export(bookTheme)
export(easyBoundaryFunc)
export(getPackages)
export(permuteRelief)
export(quadBoundaryFunc)
export(scriptLocation)
export(transparentTheme)
